#ifndef QEXCHANGE_H
#define QEXCHANGE_H

#include <QtCore>
#include <QtGui>
#include <QtNetwork>
#include <QtXml>
#include <QtSql>

#define APPLICATION_NAME "QExchange"
#define APPLICATION_VERSION "1.0.0"

#include "xthttp.h"

class QExchange : public QApplication
{
  Q_OBJECT

  int downloadStep;
  QSqlDatabase db;
  QPointer<QTemporaryFile> currentXmlFile;
  XtHttpStreamDownload ecbHttp;
  QMap<QString,QVariant> mainSettings;

  void loadSettings();
  void saveSettings();

private slots:
  void newRatesAvailable(bool error);
  void downloadStatus(int tries, int done, int total);

public:
  QExchange(int argc, char **argv);
  virtual ~QExchange();

  qreal exchange(qreal value, const QDate &atDate = QDate());

  QMap<int,qreal> rateHistory(const QString &from, const QString &to);  

  bool hasData(const QString currency = "");

  QDate minimumExchangeDate(const QString currency = "");
  QDate maximumExchangeDate(const QString currency = "");
  QList<QDate> ratesDates();
  QMap<QString,qreal> ratesAtDate(const QDate &atDate = QDate());

  QMap<QString,QString> currencies();

  QString fromCurrency() const
    { return mainSettings["lastFromCurrency"].toString(); }

  QString toCurrency() const
    { return mainSettings["lastToCurrency"].toString(); }

  QMap<QString,QVariant> settings() const
    { return mainSettings; }

  void setNewSettings(const QMap<QString,QVariant> &newSettings);

signals:
  void startDownload();
  void downloadFailed();
  void downloadCompleted();
  void downloadInProgress(int downloadStep, int offset, int total);
  void storeInProgress(int storeStep, int offset, int total);
  void changed();

public slots:
  void update();
  void setFromCurrency(const QString &currency);
  void setToCurrency(const QString &currency);
};

#endif
