#ifndef XTHTTP_H
#define XTHTTP_H

#include <QtCore>
#include <QtNetwork>

#define DEFAULT_TIMEOUT 30000
#define DEFAULT_TRIES 5

class XtHttp : public QHttp
{
  Q_OBJECT

  bool _timedOut;
  QTimer httpTimeout;

  void init();

private slots:
  void hasTimedOut();
  void stateHasChanged(int state);

public:
  XtHttp(QObject *parent = 0) : QHttp(parent)
    { init(); }

  XtHttp(const QString &hostName, quint16 port = 80, QObject *parent = 0) :
    QHttp(hostName,port,parent)
    { init(); }

  XtHttp(const QString &hostName, ConnectionMode mode, quint16 port = 0, QObject *parent = 0) :
    QHttp(hostName,mode,port,parent)
    { init(); }

  void setTimeout(int time_ms)
    { httpTimeout.setInterval(time_ms); }

  int timeout() const
    { return httpTimeout.interval(); }

  bool timedOut() const
    { return _timedOut; }
};

class XtHttpStreamDownload : public QObject
{
  Q_OBJECT

  int _tries;
  int _triesCount;
  int _current;
  int _total;
  int _nowAt;
  QIODevice *currentDevice;
  XtHttp *http;
  QUrl currentUrl;
  QNetworkProxy currentProxy;

  XtHttp *newHttp();
  void requestDownload();

private slots:
  void dataIsComing(int done, int total);
  void downloadDone(bool error);

public:
  XtHttpStreamDownload(QObject *parent = 0) : QObject(parent), _tries(DEFAULT_TRIES) { }

  void setTries(int numTries)
    { _tries = numTries; }

  int tries() const
    { return _tries; }

  void setProxy(const QNetworkProxy &proxy)
    { currentProxy = proxy; }

  void download(const QUrl &url, QIODevice *to);

signals:
  void downloadProgress(int currentTry, int done, int total);
  void downloadCompleted(bool error);
};

#endif
