#include <QtCore>
#include <QtNetwork>

#include "xthttp.h"

void XtHttp::init()
{
  httpTimeout.setSingleShot(true);
  httpTimeout.setInterval(DEFAULT_TIMEOUT);
  connect(&httpTimeout,SIGNAL(timeout()),this,SLOT(hasTimedOut()));
  connect(this,SIGNAL(dataReadProgress(int,int)),&httpTimeout,SLOT(start()));
  connect(this,SIGNAL(dataSendProgress(int,int)),&httpTimeout,SLOT(start()));

  connect(this,SIGNAL(stateChanged(int)),this,SLOT(stateHasChanged(int)));
}

void XtHttp::stateHasChanged(int state)
{
  switch (state)
  {
    case QHttp::Connected:
    case QHttp::Unconnected:
      httpTimeout.stop();
      break;

    case QHttp::HostLookup:
    case QHttp::Connecting:
      _timedOut = false;
      /* break; */

    case QHttp::Sending:
    case QHttp::Closing:
    case QHttp::Reading:
      httpTimeout.start();
      break;

    default:
      break;
  }
}

void XtHttp::hasTimedOut()
{
  _timedOut = true;
  abort();
}

XtHttp *XtHttpStreamDownload::newHttp()
{
  XtHttp *result = new XtHttp(parent());

  result->setProxy(currentProxy);
  connect(result,SIGNAL(dataReadProgress(int,int)),this,SLOT(dataIsComing(int,int)));
  connect(result,SIGNAL(done(bool)),this,SLOT(downloadDone(bool)));

  return result;
}

void XtHttpStreamDownload::download(const QUrl &url, QIODevice *to)
{
  currentDevice = to;
  currentUrl = url;
  _current = _total = 0;
  _triesCount = 0;

  requestDownload();
}

void XtHttpStreamDownload::requestDownload()
{
  QHttpRequestHeader header("GET",currentUrl.path());

  http = newHttp();
  http->setHost(currentUrl.host());

  _nowAt = 0;
  header.setValue("Host",currentUrl.host());

  if (_total)
    header.setValue("Range",QString("bytes=%1-%2").arg(_current).arg(_total));

  http->request(header,0,currentDevice);
}

void XtHttpStreamDownload::dataIsComing(int done, int total)
{
  if (!_total)
    _total = total;

  _nowAt = done;
  emit downloadProgress(_triesCount,_current + done,_total);
}

void XtHttpStreamDownload::downloadDone(bool error)
{
  if (error)
    if ((++_triesCount == _tries) || (!http->timedOut()))
      emit downloadCompleted(true);
    else
    {
      _current += _nowAt + 1;
      http->deleteLater();
      requestDownload();
    }
  else
    emit downloadCompleted(false);
}
